#!/usr/bin/env node

var connect = require("connect");
var http = require("http");
var querystring = require("querystring");
var parse = require('url').parse;
var os = require('os');


var thrift = require('mmikulicic-thrift'),
    IndexService = require('./gen-nodejs/IndexService'),
    ttypes = require('./gen-nodejs/index_types');


function createClient() {
    var connection = thrift.createConnection("localhost", 9090);
    return thrift.createClient(IndexService, connection);

}


var clients = [];
var cpus = os.cpus().length;
for(var i=0; i<cpus; i++)
    clients.push(createClient());
var currentClient = 0;

function getClient() {
    return clients[currentClient++ % clients.length];
}

function reply(res) {
    return function(err, response) {
        res.writeHead(200, {"Content-Type": "text/xml; charset=utf-8"});
        if(err) 
            res.end(err.why);
        else
            res.end(response);
    }
}

function params(req) {
    return querystring.parse(parse(req.url).query);
}

function indexLookup(req, res, next) {
    var p = querystring.parse(parse(req.url).query)
    
    getClient().indexLookup(p.id, p.query, p.mdformat, p.layout, reply(res));
}

function getBrowsingStatistics(req, res, next) {
    var p = params(req);
    
    getClient().getBrowsingStatistics(p.query, p.index, p.mdformat, p.layout, reply(res));
}


function getNumberOfElements(req, res, next) {
    var p = params(req);

    getClient().getNumberOfElements(p.rsId, reply(res));
};

function getResult(req, res, next) {
    var p = params(req);

    getClient().getResult(p.rsId, p.fromPosition, p.toPosition, reply(res));
};

connect(
    connect.router(function(app) {
        app.get('/IndexService/indexLookup',  indexLookup);
        app.get('/IndexService/getBrowsingStatistics',  getBrowsingStatistics);
        app.get('/resultSet/getNumberOfElements', getNumberOfElements);
        app.get('/resultSet/getResult', getResult);

        app.get('/:context/services/IndexService/indexLookup',  indexLookup);
        app.get('/:context/services/IndexService/getBrowsingStatistics',  getBrowsingStatistics);
        app.get('/:context/services/resultSet/getNumberOfElements', getNumberOfElements);
        app.get('/:context/services/resultSet/getResult', getResult);

        app.get('/:context/mvc/directindex.do/IndexService/indexLookup',  indexLookup);
        app.get('/:context/mvc/directindex.do/IndexService/getBrowsingStatistics',  getBrowsingStatistics);
        app.get('/:context/mvc/directindex.do/resultSet/getNumberOfElements', getNumberOfElements);
        app.get('/:context/mvc/directindex.do/resultSet/getResult', getResult);

    })
).listen(3000);
